[comment {-*- tcl -*- doctool manpage}]
[manpage_begin config n 0.1]
[copyright {2006 Georg Lehner <jorge@magma.com.ni}]
[moddesc   {options managment}]
[titledesc {onfiguration and preferences managment}]
[require Tcl 8.2]
[require config [opt 0.1]]

[description]
[para]

The [package config] package provides flexible options managment.
[para]

A programms configuration or user preferences can be stored in
[term optionset]s, which can be set from configuration files, the
environment or from the commandline.



[section OVERVIEW]

[para]
[term option]s are elements of [term optionset]s, their value can be set from
several [term source]s.

An [term optionset] holds information about [term source]s, default
values, descriptions, shortcuts, configuration file locations of a
range of [term options]

[para]
In the end, an [term optionset] is a Tcl array which can be
manipulated as needed by the program.


[para]

Configuration files can be specified explicitely, else an internal
heuristic can be used to determine posible locations and names of
configuration files.  Any number of configuration files can be
specified for an [term optionset].

The format of each configuration file can be defined individually.
Actually (safe) Tcl is the default and actually the only option.

[para]

Commandline parsing is also configurable, beeing the [package cmdline]
package the default and actually only option.

[para]
 Note that the the [package cmdline] package is not required, if
 commandline parseing is not used.


[para]
Although [package config] is designed with extensibility in mind, its
main purpose is to simplify option handling.  The default way to
invoke option processing is just to call [cmd ::config::get_config].
[para]
The following sequence is then invoked to process options:

[list_begin enum]
[enum]
determine location of configuration files in the following order:
[list_begin bullet]
[bullet] from program defaults
[bullet] from the environment
[bullet] from the commandline
[list_end]
[enum]

The following stages are processed to determine the value of a all
other options:
[list_begin bullet]
[bullet] program default
[bullet] system
[bullet] user
[bullet] environment
[bullet] commandline
[list_end]
[list_end]

if a value is obtained from a later stage, it overrides any data
collected from an earlier stage:


[section COMMANDS]

[subsection VARIABLES]
[list_begin definitions]
[lst_item [var ::config::optionsets]]  list of defined optionsets
[lst_item [var ::config::version]]  version of the config package
[list_end]

[subsection {Defining Optionsets}]


[list_begin definitions]
[call [cmd ::config::add_option] [arg optionset] [arg name]\
     [arg short_description] [opt options]]

Define an optionset. posible [opt options] are:

[list_begin opt]
[opt_def -secret] don't show the option to the user

[opt_def -optional] this option may have an argument

[opt_def -default defaultvalue] set the default value

[opt_def -shortcut shortcut]

the first letter of [const shortcut] is used as sinlge letter option
name for the option

[opt_def -env envname]

[const envname] is the name of the environment variable used to set
the option

[list_end]
[nl]


[call [cmd ::config::add_file] [arg optionset] [opt [arg parser]]\
     [opt [arg {-tag tag}]] [opt [arg {filename ..}]]]

Register a configuration file to be parsed with [arg parser]. Eventuall add
a [arg tag] to distinguish configuration file setup. The list of
files: [arg {filename ..}] will be searched in order, the first file matched is
used. 

[nl]

If the list of files is empty, search in several places with
predefined names derived from [var argv0] and [arg tag].  If [arg tag]
is not specified set it to [const {$argv0}].  If [arg parser] is not
specified set it to [const tcl]

[list_end]


[subsection {Get the options set}]

[list_begin definitions]

[call [cmd ::config::get_config] [arg optionset] [opt [arg parser]]\
     [opt [arg usage]] [opt [arg envname]] [opt [arg shortcut]]]

  parse options from environment, config files and the commandline of
  a program.

[list_begin arg]
[arg_def string usage]  modfiy the usage message of cmdline
[arg_def string shortcut]  modify the shortcut for the configfile option, default 'c'
[arg_def token envname]  modify the environment variable used to look up the
            configfile, default CONFIG
[arg_def token parser]  modify the parser used to parse the configfile, default tcl
[list_end]

[call [cmd ::config::env_getoptions] [arg optionset]]

set options from environment variables

 
[call [cmd ::config::parse_file] [arg optionset] [opt [arg tag]]]

  (eventually) set options by parsing the configuration file specified
by [arg tag], or the default configuration file if [arg tag] is not specified

[call [cmd ::config::tcl_parse_file] [arg optionset] [arg filename]]

parse [arg filename] in a safe interpreter and set [arg optioneset] with
variables defined there whose name is in [const {$optionset(options)}].

[call [cmd ::config::cmdline_getoptions] [arg  optionset] [arg argv]\
     [opt [arg usage]]]

parse the argumentstring in the variable named [arg argv] into the optionset
with the [package cmdline] parser.

[call [cmd ::config::optionset] [arg optionset]\
     [opt [arg processor]]]
 
[list_begin arg]
[arg_def token optionset] name of an array holding optionset definition
[arg_def token processor] name of the processor to use, by default
[package cmdline]
[list_end]

The [arg optionset] is converted into a form usable by the [arg processor] and
then [const return]'ed

[list_end]


[section {Terms and Concepts}]

[list_begin definitions]
[lst_item option]

An option is some data that determines certain behavior of a program

[lst_item configuration]

is an option that is determined at programm start and is not to be
changed during the runtime of a programm.

[lst_item preference]

is an option that is initialized during program start, may be altered
during the runtime of a program and may be saved for future
invocations of the program.
[list_end]


[subsection SCOPE]

[list_begin definitions]

[lst_item {system (wide setting)}]

Is data for options, that is given for invocations under any of the
systems users.

[lst_item {user (setting)}]

Is data for options, that is specific for invocation of a specific
user.

[lst_item {programm (default setting)}]

is data for options, that is predefined for invocation of the program
on any system.
[list_end]


[subsection SOURCES]

configuration options can be set from:

[list_begin bullet]
[bullet] files
[bullet] environment variables
[bullet] commandline options and arguments
[list_end]

preferences can also be set from program input (user interaction) 


[subsection OPTIONSETs]

In the following * means not supported by [package cmdline] package

[para]
An optionset is an array, describing options.  The optionset is first
defined, and can then be manipulated by searching the environment, the
commandline, several configuration or preferences files and user input
during the runtime of the program.

[para]
In the following list of reserved indexes 'option' containes the name
of an option

[list_begin opt]
[opt_def {($option)}]
  holds the value of the option after option processing

[opt_def (options)] list of option names, auto generated by [cmd add_option]

[opt_def (shortcuts)] list of the respective shortcuts
  No check is made to garantee unique shortcuts

[opt_def {($option,shortcut)} token]
[list_begin opt]
[opt_def string]  the letter to use is the first letter of string
[opt_def {true or unset}]  use first letter of longname
[opt_def false]  do not define a short option *
[list_end]

[opt_def {($shortcut,option)} option]
  this reverse index of shortcuts to optionname is created during
  parsing of the options, put deleted afterwards

[opt_def {($option,secret)} flag]
[list_begin opt]
[opt_def true]  do not show the option to the user
[opt_def {false or unset}]  show in usage messages, etc.
when using [package cmdline] this is made a boolean option
[list_end]

[opt_def {($option,default)} defaultvalue]
[list_begin opt]
[opt_def {not set}]  option is a boolean
[opt_def else]  value to give the option if not defined elsewhere
[list_end]

[opt_def {($options,optional)} flag]
  only taken into account if defaultvalue exists

[list_begin opt]
[opt_def true]  the option may be specified without an argument
in which case it is set to the [arg defaultvalue] *
[opt_def not {existent or false}]  an argument to the option must be given
[list_end]


[opt_def {($option,description)} "long description"] *

[opt_def {($option,short)} "(less then) one line description"]
this index is [emph required]

[opt_def {($option,env)} name]
  name of the asociated environment variable

[opt_def (tags)]  list of tags, each describing a configuration fileset
[opt_def {($tag,files)}] list of possible configuration files
[opt_def {($tag,parser)}] parser used for this configuration file
[opt_def {($tag,filename)}] first filename out of [const {($tag,files)}] which exists

If no tag is specified:
[opt_def (files)] list of files 
[opt_def (filename)] first filename out of (files)
[opt_def (parser)] parser to use for (filename)

[list_end]



[section EXAMPLES]

[example_begin]
package require config
::config::add_options config <name> <description> \
                             -default <value> -env <NAME>
..
::config::get_config config
[example_end]


This will set [const {$config(<name>)}] to one of:
[list_begin bullet]
[bullet]
  <value>
[bullet]
 the value of the variable <name> set in a tcl config file that
   can be placed at various system wide or user specific places
[bullet]
 the value of the environment variable <NAME>
[bullet]
 the value of the -<n> switch on the commandline, where <n> is the
   first letter of <name>
[list_end]

Later items in the list precede earlier items.



[section {SEE ALSO}]

[list_begin bullet]
[bullet]
[uri http://www.magma.com.ni/moin/TclConfig\
     "Author's Webpage about [package config]"]

[bullet]
[uri http://tcllib.sourceforge.net/doc/cmdline.html\
     "[package cmdline] package"]

[bullet]
[uri http://wiki.tcl.tk/2438 {Tcler's Wiki about configuration file handling}]
[list_end]

[keywords {cmdline processing} {argument processing} {configuration files}]

[manpage_end]